@@ -24,41 +24,6 @@
 
 hd <- function(x, series = 1, transition = 0){
 
-  # Function to calculate matrix potence
-  "%^%" <- function(A, n){
-    if(n == 1){
-      A
-    }else{
-      A %*% (A %^% (n-1))
-    }
-  }
-
-
-  # function to calculate impulse response
-  IrF <- function(A_hat, B_hat, horizon){
-    k <- nrow(A_hat)
-    p <- ncol(A_hat)/k
-    if(p == 1){
-      irfa <- array(0, c(k, k, horizon))
-      irfa[,,1] <- B_hat
-      for(i in 1:horizon){
-        irfa[,,i] <- (A_hat%^%i)%*%B_hat
-      }
-      return(irfa)
-    }else{
-      irfa <- array(0, c(k, k, horizon))
-      irfa[,,1] <- B_hat
-      Mm <- matrix(0, nrow = k*p, ncol = k*p)
-      Mm[1:k, 1:(k*p)] <- A_hat
-      Mm[(k+1):(k*p), 1 : ((p-1)*k)] <- diag(k*(p-1))
-      Mm1 <- diag(k*p)
-      for(i in 1:(horizon-1)){
-        Mm1 <- Mm1%*%Mm
-        irfa[,,(i+1)] <- Mm1[1:k, 1:k]%*%B_hat
-      }
-      return(irfa)
-    }
-  }
 
   ## Step 1: Calculate MA coefficients
 
@@ -76,7 +41,7 @@ hd <- function(x, series = 1, transition = 0){
 
   horizon <- x$n
 
-  IR <- IrF(A_hat, B_hat, horizon)
+  IR <- array(unlist(IRF(A_hat, B_hat, horizon)), c(x$K, x$K, horizon))
 
   impulse <- matrix(0, ncol = dim(IR)[2]^2 + 1, nrow = dim(IR)[3])
   colnames(impulse) <- rep("V1", ncol(impulse))
@@ -151,7 +116,7 @@ hd <- function(x, series = 1, transition = 0){
   colnames(yhat)[2] <- paste("Demeaned series ", colnames(y)[series])
 
   for(i in 4:ncol(yhat)){
-    colnames(yhat)[i] <- paste("Cumulative effect of flow ", colnames(y)[i-3], "shock on ", colnames(y)[series])
+    colnames(yhat)[i] <- paste("Cumulative effect of ", colnames(y)[i-3], "shock on ", colnames(y)[series])
   }
 
   if(inherits(x$y, "ts")){
